#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ann4qa
# @Time         : 2023/4/24 18:10
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from chatllm.applications import ChatBase
from chatllm.embedding import SentenceEmbedding


class ChatANN(ChatBase):

    def __init__(self, backend='docarray', encode_model="shibing624/text2vec-base-chinese", **kwargs):
        super().__init__(**kwargs)
        self.backend = backend  # todo 增加更多的ann后端

        self.encode = SentenceEmbedding(encode_model).encode  # 加缓存

        # create_ann_index
        self.ann_index = None

    def qa(self, query, topk=5, **kwargs):
        df = self.ann_find(query, topk)
        knowledge_base = '\n'.join(df.text)

        return self._qa(query, knowledge_base, **kwargs)

    def ann_find(self, query, topk=5):  # 返回df
        v = self.encode(query)[0]
        if self.backend == 'docarray':
            result = self.ann_index.find(v, limit=topk)[:, ('text', 'scores__cosine__value')]
            df = pd.DataFrame(zip(*result[:, ('id', 'text', 'scores__cosine__value')]))
            df.columns = ('id', 'text', 'score')
            df['score'] = 1 - df['score']
            return df  # 单向量

    def create_ann_index(self, texts):
        self.ann_index = self.encode(texts, show_progress_bar=True, return_document=True)


if __name__ == '__main__':

    qa = ChatANN(chat_func=None)
    qa.load_llm4chat(model_name_or_path="/Users/betterme/PycharmProjects/AI/CHAT_MODEL/chatglm")

    for i, _ in qa(query='1+1'):
        pass
